#define _CFFI_

/* We try to define Py_LIMITED_API before including Python.h.

   Mess: we can only define it if Py_DEBUG, Py_TRACE_REFS and
   Py_REF_DEBUG are not defined.  This is a best-effort approximation:
   we can learn about Py_DEBUG from pyconfig.h, but it is unclear if
   the same works for the other two macros.  Py_DEBUG implies them,
   but not the other way around.

   The implementation is messy (issue #350): on Windows, with _MSC_VER,
   we have to define Py_LIMITED_API even before including pyconfig.h.
   In that case, we guess what pyconfig.h will do to the macros above,
   and check our guess after the #include.

   Note that on Windows, with CPython 3.x, you need >= 3.5 and virtualenv
   version >= 16.0.0.  With older versions of either, you don't get a
   copy of PYTHON3.DLL in the virtualenv.  We can't check the version of
   CPython *before* we even include pyconfig.h.  ffi.set_source() puts
   a ``#define _CFFI_NO_LIMITED_API'' at the start of this file if it is
   running on Windows < 3.5, as an attempt at fixing it, but that's
   arguably wrong because it may not be the target version of Python.
   Still better than nothing I guess.  As another workaround, you can
   remove the definition of Py_LIMITED_API here.

   See also 'py_limited_api' in cffi/setuptools_ext.py.
*/
#if !defined(_CFFI_USE_EMBEDDING) && !defined(Py_LIMITED_API)
#  ifdef _MSC_VER
#    if !defined(_DEBUG) && !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#    include <pyconfig.h>
     /* sanity-check: Py_LIMITED_API will cause crashes if any of these
        are also defined.  Normally, the Python file PC/pyconfig.h does not
        cause any of these to be defined, with the exception that _DEBUG
        causes Py_DEBUG.  Double-check that. */
#    ifdef Py_LIMITED_API
#      if defined(Py_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_DEBUG, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_TRACE_REFS)
#        error "pyconfig.h unexpectedly defines Py_TRACE_REFS, but Py_LIMITED_API is set"
#      endif
#      if defined(Py_REF_DEBUG)
#        error "pyconfig.h unexpectedly defines Py_REF_DEBUG, but Py_LIMITED_API is set"
#      endif
#    endif
#  else
#    include <pyconfig.h>
#    if !defined(Py_DEBUG) && !defined(Py_TRACE_REFS) && !defined(Py_REF_DEBUG) && !defined(_CFFI_NO_LIMITED_API)
#      define Py_LIMITED_API
#    endif
#  endif
#endif

#include <Python.h>
#ifdef __cplusplus
extern "C" {
#endif
#include <stddef.h>

/* This part is from file 'cffi/parse_c_type.h'.  It is copied at the
   beginning of C sources generated by CFFI's ffi.set_source(). */

typedef void *_cffi_opcode_t;

#define _CFFI_OP(opcode, arg)   (_cffi_opcode_t)(opcode | (((uintptr_t)(arg)) << 8))
#define _CFFI_GETOP(cffi_opcode)    ((unsigned char)(uintptr_t)cffi_opcode)
#define _CFFI_GETARG(cffi_opcode)   (((intptr_t)cffi_opcode) >> 8)

#define _CFFI_OP_PRIMITIVE       1
#define _CFFI_OP_POINTER         3
#define _CFFI_OP_ARRAY           5
#define _CFFI_OP_OPEN_ARRAY      7
#define _CFFI_OP_STRUCT_UNION    9
#define _CFFI_OP_ENUM           11
#define _CFFI_OP_FUNCTION       13
#define _CFFI_OP_FUNCTION_END   15
#define _CFFI_OP_NOOP           17
#define _CFFI_OP_BITFIELD       19
#define _CFFI_OP_TYPENAME       21
#define _CFFI_OP_CPYTHON_BLTN_V 23   // varargs
#define _CFFI_OP_CPYTHON_BLTN_N 25   // noargs
#define _CFFI_OP_CPYTHON_BLTN_O 27   // O  (i.e. a single arg)
#define _CFFI_OP_CONSTANT       29
#define _CFFI_OP_CONSTANT_INT   31
#define _CFFI_OP_GLOBAL_VAR     33
#define _CFFI_OP_DLOPEN_FUNC    35
#define _CFFI_OP_DLOPEN_CONST   37
#define _CFFI_OP_GLOBAL_VAR_F   39
#define _CFFI_OP_EXTERN_PYTHON  41

#define _CFFI_PRIM_VOID          0
#define _CFFI_PRIM_BOOL          1
#define _CFFI_PRIM_CHAR          2
#define _CFFI_PRIM_SCHAR         3
#define _CFFI_PRIM_UCHAR         4
#define _CFFI_PRIM_SHORT         5
#define _CFFI_PRIM_USHORT        6
#define _CFFI_PRIM_INT           7
#define _CFFI_PRIM_UINT          8
#define _CFFI_PRIM_LONG          9
#define _CFFI_PRIM_ULONG        10
#define _CFFI_PRIM_LONGLONG     11
#define _CFFI_PRIM_ULONGLONG    12
#define _CFFI_PRIM_FLOAT        13
#define _CFFI_PRIM_DOUBLE       14
#define _CFFI_PRIM_LONGDOUBLE   15

#define _CFFI_PRIM_WCHAR        16
#define _CFFI_PRIM_INT8         17
#define _CFFI_PRIM_UINT8        18
#define _CFFI_PRIM_INT16        19
#define _CFFI_PRIM_UINT16       20
#define _CFFI_PRIM_INT32        21
#define _CFFI_PRIM_UINT32       22
#define _CFFI_PRIM_INT64        23
#define _CFFI_PRIM_UINT64       24
#define _CFFI_PRIM_INTPTR       25
#define _CFFI_PRIM_UINTPTR      26
#define _CFFI_PRIM_PTRDIFF      27
#define _CFFI_PRIM_SIZE         28
#define _CFFI_PRIM_SSIZE        29
#define _CFFI_PRIM_INT_LEAST8   30
#define _CFFI_PRIM_UINT_LEAST8  31
#define _CFFI_PRIM_INT_LEAST16  32
#define _CFFI_PRIM_UINT_LEAST16 33
#define _CFFI_PRIM_INT_LEAST32  34
#define _CFFI_PRIM_UINT_LEAST32 35
#define _CFFI_PRIM_INT_LEAST64  36
#define _CFFI_PRIM_UINT_LEAST64 37
#define _CFFI_PRIM_INT_FAST8    38
#define _CFFI_PRIM_UINT_FAST8   39
#define _CFFI_PRIM_INT_FAST16   40
#define _CFFI_PRIM_UINT_FAST16  41
#define _CFFI_PRIM_INT_FAST32   42
#define _CFFI_PRIM_UINT_FAST32  43
#define _CFFI_PRIM_INT_FAST64   44
#define _CFFI_PRIM_UINT_FAST64  45
#define _CFFI_PRIM_INTMAX       46
#define _CFFI_PRIM_UINTMAX      47
#define _CFFI_PRIM_FLOATCOMPLEX 48
#define _CFFI_PRIM_DOUBLECOMPLEX 49
#define _CFFI_PRIM_CHAR16       50
#define _CFFI_PRIM_CHAR32       51

#define _CFFI__NUM_PRIM         52
#define _CFFI__UNKNOWN_PRIM           (-1)
#define _CFFI__UNKNOWN_FLOAT_PRIM     (-2)
#define _CFFI__UNKNOWN_LONG_DOUBLE    (-3)

#define _CFFI__IO_FILE_STRUCT         (-1)


struct _cffi_global_s {
    const char *name;
    void *address;
    _cffi_opcode_t type_op;
    void *size_or_direct_fn;  // OP_GLOBAL_VAR: size, or 0 if unknown
                              // OP_CPYTHON_BLTN_*: addr of direct function
};

struct _cffi_getconst_s {
    unsigned long long value;
    const struct _cffi_type_context_s *ctx;
    int gindex;
};

struct _cffi_struct_union_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_STRUCT_UNION
    int flags;               // _CFFI_F_* flags below
    size_t size;
    int alignment;
    int first_field_index;   // -> _cffi_fields array
    int num_fields;
};
#define _CFFI_F_UNION         0x01   // is a union, not a struct
#define _CFFI_F_CHECK_FIELDS  0x02   // complain if fields are not in the
                                     // "standard layout" or if some are missing
#define _CFFI_F_PACKED        0x04   // for CHECK_FIELDS, assume a packed struct
#define _CFFI_F_EXTERNAL      0x08   // in some other ffi.include()
#define _CFFI_F_OPAQUE        0x10   // opaque

struct _cffi_field_s {
    const char *name;
    size_t field_offset;
    size_t field_size;
    _cffi_opcode_t field_type_op;
};

struct _cffi_enum_s {
    const char *name;
    int type_index;          // -> _cffi_types, on a OP_ENUM
    int type_prim;           // _CFFI_PRIM_xxx
    const char *enumerators; // comma-delimited string
};

struct _cffi_typename_s {
    const char *name;
    int type_index;   /* if opaque, points to a possibly artificial
                         OP_STRUCT which is itself opaque */
};

struct _cffi_type_context_s {
    _cffi_opcode_t *types;
    const struct _cffi_global_s *globals;
    const struct _cffi_field_s *fields;
    const struct _cffi_struct_union_s *struct_unions;
    const struct _cffi_enum_s *enums;
    const struct _cffi_typename_s *typenames;
    int num_globals;
    int num_struct_unions;
    int num_enums;
    int num_typenames;
    const char *const *includes;
    int num_types;
    int flags;      /* future extension */
};

struct _cffi_parse_info_s {
    const struct _cffi_type_context_s *ctx;
    _cffi_opcode_t *output;
    unsigned int output_size;
    size_t error_location;
    const char *error_message;
};

struct _cffi_externpy_s {
    const char *name;
    size_t size_of_result;
    void *reserved1, *reserved2;
};

#ifdef _CFFI_INTERNAL
static int parse_c_type(struct _cffi_parse_info_s *info, const char *input);
static int search_in_globals(const struct _cffi_type_context_s *ctx,
                             const char *search, size_t search_len);
static int search_in_struct_unions(const struct _cffi_type_context_s *ctx,
                                   const char *search, size_t search_len);
#endif

/* this block of #ifs should be kept exactly identical between
   c/_cffi_backend.c, cffi/vengine_cpy.py, cffi/vengine_gen.py
   and cffi/_cffi_include.h */
#if defined(_MSC_VER)
# include <malloc.h>   /* for alloca() */
# if _MSC_VER < 1600   /* MSVC < 2010 */
   typedef __int8 int8_t;
   typedef __int16 int16_t;
   typedef __int32 int32_t;
   typedef __int64 int64_t;
   typedef unsigned __int8 uint8_t;
   typedef unsigned __int16 uint16_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
   typedef __int8 int_least8_t;
   typedef __int16 int_least16_t;
   typedef __int32 int_least32_t;
   typedef __int64 int_least64_t;
   typedef unsigned __int8 uint_least8_t;
   typedef unsigned __int16 uint_least16_t;
   typedef unsigned __int32 uint_least32_t;
   typedef unsigned __int64 uint_least64_t;
   typedef __int8 int_fast8_t;
   typedef __int16 int_fast16_t;
   typedef __int32 int_fast32_t;
   typedef __int64 int_fast64_t;
   typedef unsigned __int8 uint_fast8_t;
   typedef unsigned __int16 uint_fast16_t;
   typedef unsigned __int32 uint_fast32_t;
   typedef unsigned __int64 uint_fast64_t;
   typedef __int64 intmax_t;
   typedef unsigned __int64 uintmax_t;
# else
#  include <stdint.h>
# endif
# if _MSC_VER < 1800   /* MSVC < 2013 */
#  ifndef __cplusplus
    typedef unsigned char _Bool;
#  endif
# endif
# define _cffi_float_complex_t   _Fcomplex    /* include <complex.h> for it */
# define _cffi_double_complex_t  _Dcomplex    /* include <complex.h> for it */
#else
# include <stdint.h>
# if (defined (__SVR4) && defined (__sun)) || defined(_AIX) || defined(__hpux)
#  include <alloca.h>
# endif
# define _cffi_float_complex_t   float _Complex
# define _cffi_double_complex_t  double _Complex
#endif

#ifdef __GNUC__
# define _CFFI_UNUSED_FN  __attribute__((unused))
#else
# define _CFFI_UNUSED_FN  /* nothing */
#endif

#ifdef __cplusplus
# ifndef _Bool
   typedef bool _Bool;   /* semi-hackish: C++ has no _Bool; bool is builtin */
# endif
#endif

/**********  CPython-specific section  **********/
#ifndef PYPY_VERSION


#if PY_MAJOR_VERSION >= 3
# define PyInt_FromLong PyLong_FromLong
#endif

#define _cffi_from_c_double PyFloat_FromDouble
#define _cffi_from_c_float PyFloat_FromDouble
#define _cffi_from_c_long PyInt_FromLong
#define _cffi_from_c_ulong PyLong_FromUnsignedLong
#define _cffi_from_c_longlong PyLong_FromLongLong
#define _cffi_from_c_ulonglong PyLong_FromUnsignedLongLong
#define _cffi_from_c__Bool PyBool_FromLong

#define _cffi_to_c_double PyFloat_AsDouble
#define _cffi_to_c_float PyFloat_AsDouble

#define _cffi_from_c_int(x, type)                                        \
    (((type)-1) > 0 ? /* unsigned */                                     \
        (sizeof(type) < sizeof(long) ?                                   \
            PyInt_FromLong((long)x) :                                    \
         sizeof(type) == sizeof(long) ?                                  \
            PyLong_FromUnsignedLong((unsigned long)x) :                  \
            PyLong_FromUnsignedLongLong((unsigned long long)x)) :        \
        (sizeof(type) <= sizeof(long) ?                                  \
            PyInt_FromLong((long)x) :                                    \
            PyLong_FromLongLong((long long)x)))

#define _cffi_to_c_int(o, type)                                          \
    ((type)(                                                             \
     sizeof(type) == 1 ? (((type)-1) > 0 ? (type)_cffi_to_c_u8(o)        \
                                         : (type)_cffi_to_c_i8(o)) :     \
     sizeof(type) == 2 ? (((type)-1) > 0 ? (type)_cffi_to_c_u16(o)       \
                                         : (type)_cffi_to_c_i16(o)) :    \
     sizeof(type) == 4 ? (((type)-1) > 0 ? (type)_cffi_to_c_u32(o)       \
                                         : (type)_cffi_to_c_i32(o)) :    \
     sizeof(type) == 8 ? (((type)-1) > 0 ? (type)_cffi_to_c_u64(o)       \
                                         : (type)_cffi_to_c_i64(o)) :    \
     (Py_FatalError("unsupported size for type " #type), (type)0)))

#define _cffi_to_c_i8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[1])
#define _cffi_to_c_u8                                                    \
                 ((int(*)(PyObject *))_cffi_exports[2])
#define _cffi_to_c_i16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[3])
#define _cffi_to_c_u16                                                   \
                 ((int(*)(PyObject *))_cffi_exports[4])
#define _cffi_to_c_i32                                                   \
                 ((int(*)(PyObject *))_cffi_exports[5])
#define _cffi_to_c_u32                                                   \
                 ((unsigned int(*)(PyObject *))_cffi_exports[6])
#define _cffi_to_c_i64                                                   \
                 ((long long(*)(PyObject *))_cffi_exports[7])
#define _cffi_to_c_u64                                                   \
                 ((unsigned long long(*)(PyObject *))_cffi_exports[8])
#define _cffi_to_c_char                                                  \
                 ((int(*)(PyObject *))_cffi_exports[9])
#define _cffi_from_c_pointer                                             \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[10])
#define _cffi_to_c_pointer                                               \
    ((char *(*)(PyObject *, struct _cffi_ctypedescr *))_cffi_exports[11])
#define _cffi_get_struct_layout                                          \
    not used any more
#define _cffi_restore_errno                                              \
    ((void(*)(void))_cffi_exports[13])
#define _cffi_save_errno                                                 \
    ((void(*)(void))_cffi_exports[14])
#define _cffi_from_c_char                                                \
    ((PyObject *(*)(char))_cffi_exports[15])
#define _cffi_from_c_deref                                               \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[16])
#define _cffi_to_c                                                       \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[17])
#define _cffi_from_c_struct                                              \
    ((PyObject *(*)(char *, struct _cffi_ctypedescr *))_cffi_exports[18])
#define _cffi_to_c_wchar_t                                               \
    ((_cffi_wchar_t(*)(PyObject *))_cffi_exports[19])
#define _cffi_from_c_wchar_t                                             \
    ((PyObject *(*)(_cffi_wchar_t))_cffi_exports[20])
#define _cffi_to_c_long_double                                           \
    ((long double(*)(PyObject *))_cffi_exports[21])
#define _cffi_to_c__Bool                                                 \
    ((_Bool(*)(PyObject *))_cffi_exports[22])
#define _cffi_prepare_pointer_call_argument                              \
    ((Py_ssize_t(*)(struct _cffi_ctypedescr *,                           \
                    PyObject *, char **))_cffi_exports[23])
#define _cffi_convert_array_from_object                                  \
    ((int(*)(char *, struct _cffi_ctypedescr *, PyObject *))_cffi_exports[24])
#define _CFFI_CPIDX  25
#define _cffi_call_python                                                \
    ((void(*)(struct _cffi_externpy_s *, char *))_cffi_exports[_CFFI_CPIDX])
#define _cffi_to_c_wchar3216_t                                           \
    ((int(*)(PyObject *))_cffi_exports[26])
#define _cffi_from_c_wchar3216_t                                         \
    ((PyObject *(*)(int))_cffi_exports[27])
#define _CFFI_NUM_EXPORTS 28

struct _cffi_ctypedescr;

static void *_cffi_exports[_CFFI_NUM_EXPORTS];

#define _cffi_type(index)   (                           \
    assert((((uintptr_t)_cffi_types[index]) & 1) == 0), \
    (struct _cffi_ctypedescr *)_cffi_types[index])

static PyObject *_cffi_init(const char *module_name, Py_ssize_t version,
                            const struct _cffi_type_context_s *ctx)
{
    PyObject *module, *o_arg, *new_module;
    void *raw[] = {
        (void *)module_name,
        (void *)version,
        (void *)_cffi_exports,
        (void *)ctx,
    };

    module = PyImport_ImportModule("_cffi_backend");
    if (module == NULL)
        goto failure;

    o_arg = PyLong_FromVoidPtr((void *)raw);
    if (o_arg == NULL)
        goto failure;

    new_module = PyObject_CallMethod(
        module, (char *)"_init_cffi_1_0_external_module", (char *)"O", o_arg);

    Py_DECREF(o_arg);
    Py_DECREF(module);
    return new_module;

  failure:
    Py_XDECREF(module);
    return NULL;
}


#ifdef HAVE_WCHAR_H
typedef wchar_t _cffi_wchar_t;
#else
typedef uint16_t _cffi_wchar_t;   /* same random pick as _cffi_backend.c */
#endif

_CFFI_UNUSED_FN static uint16_t _cffi_to_c_char16_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return (uint16_t)_cffi_to_c_wchar_t(o);
    else
        return (uint16_t)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char16_t(uint16_t x)
{
    if (sizeof(_cffi_wchar_t) == 2)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

_CFFI_UNUSED_FN static int _cffi_to_c_char32_t(PyObject *o)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return (int)_cffi_to_c_wchar_t(o);
    else
        return (int)_cffi_to_c_wchar3216_t(o);
}

_CFFI_UNUSED_FN static PyObject *_cffi_from_c_char32_t(unsigned int x)
{
    if (sizeof(_cffi_wchar_t) == 4)
        return _cffi_from_c_wchar_t((_cffi_wchar_t)x);
    else
        return _cffi_from_c_wchar3216_t((int)x);
}

union _cffi_union_alignment_u {
    unsigned char m_char;
    unsigned short m_short;
    unsigned int m_int;
    unsigned long m_long;
    unsigned long long m_longlong;
    float m_float;
    double m_double;
    long double m_longdouble;
};

struct _cffi_freeme_s {
    struct _cffi_freeme_s *next;
    union _cffi_union_alignment_u alignment;
};

_CFFI_UNUSED_FN static int
_cffi_convert_array_argument(struct _cffi_ctypedescr *ctptr, PyObject *arg,
                             char **output_data, Py_ssize_t datasize,
                             struct _cffi_freeme_s **freeme)
{
    char *p;
    if (datasize < 0)
        return -1;

    p = *output_data;
    if (p == NULL) {
        struct _cffi_freeme_s *fp = (struct _cffi_freeme_s *)PyObject_Malloc(
            offsetof(struct _cffi_freeme_s, alignment) + (size_t)datasize);
        if (fp == NULL)
            return -1;
        fp->next = *freeme;
        *freeme = fp;
        p = *output_data = (char *)&fp->alignment;
    }
    memset((void *)p, 0, (size_t)datasize);
    return _cffi_convert_array_from_object(p, ctptr, arg);
}

_CFFI_UNUSED_FN static void
_cffi_free_array_arguments(struct _cffi_freeme_s *freeme)
{
    do {
        void *p = (void *)freeme;
        freeme = freeme->next;
        PyObject_Free(p);
    } while (freeme != NULL);
}

/**********  end CPython-specific section  **********/
#else
_CFFI_UNUSED_FN
static void (*_cffi_call_python_org)(struct _cffi_externpy_s *, char *);
# define _cffi_call_python  _cffi_call_python_org
#endif


#define _cffi_array_len(array)   (sizeof(array) / sizeof((array)[0]))

#define _cffi_prim_int(size, sign)                                      \
    ((size) == 1 ? ((sign) ? _CFFI_PRIM_INT8  : _CFFI_PRIM_UINT8)  :    \
     (size) == 2 ? ((sign) ? _CFFI_PRIM_INT16 : _CFFI_PRIM_UINT16) :    \
     (size) == 4 ? ((sign) ? _CFFI_PRIM_INT32 : _CFFI_PRIM_UINT32) :    \
     (size) == 8 ? ((sign) ? _CFFI_PRIM_INT64 : _CFFI_PRIM_UINT64) :    \
     _CFFI__UNKNOWN_PRIM)

#define _cffi_prim_float(size)                                          \
    ((size) == sizeof(float) ? _CFFI_PRIM_FLOAT :                       \
     (size) == sizeof(double) ? _CFFI_PRIM_DOUBLE :                     \
     (size) == sizeof(long double) ? _CFFI__UNKNOWN_LONG_DOUBLE :       \
     _CFFI__UNKNOWN_FLOAT_PRIM)

#define _cffi_check_int(got, got_nonpos, expected)      \
    ((got_nonpos) == (expected <= 0) &&                 \
     (got) == (unsigned long long)expected)

#ifdef MS_WIN32
# define _cffi_stdcall  __stdcall
#else
# define _cffi_stdcall  /* nothing */
#endif

#ifdef __cplusplus
}
#endif

/************************************************************/


#if (defined(_MSC_VER) && _MSC_VER < 1600)
    typedef __int32  int32_t;
    typedef __int64  int64_t;
#else
    #include <stdint.h>
#endif
#include <postgres_ext.h>
#include <libpq-fe.h>

int PQEgetlong(int64_t *raw_res, const PGresult *res, int tup_num, int field_num) {
    char *val = PQgetvalue(res, tup_num, field_num);
    if (!val) return -1;
    sscanf(val, "%ld", (long *)raw_res);
    return 0;
}

int PQEgetint(int32_t *raw_res, const PGresult *res, int tup_num, int field_num) {
    char *val = PQgetvalue(res, tup_num, field_num);
    if (!val) return -1;
    sscanf(val, "%d", (int *)raw_res);
    return 0;
}

int PQEgetfloat(float *raw_res, const PGresult *res, int tup_num, int field_num) {
    char *val = PQgetvalue(res, tup_num, field_num);
    if (!val) return -1;
    sscanf(val, "%f", raw_res);
    return 0;
}

int PQEgetdouble(double *raw_res, const PGresult *res, int tup_num, int field_num) {
    char *val = PQgetvalue(res, tup_num, field_num);
    if (!val) return -1;
    sscanf(val, "%lf", raw_res);
    return 0;
}

// Real names start with PG_DIAG_, but here we define our prefixes,
// because some are defined and some are not depending on pg version.

static int const LIBPQ_DIAG_SEVERITY = 'S';
static int const LIBPQ_DIAG_SQLSTATE = 'C';
static int const LIBPQ_DIAG_MESSAGE_PRIMARY = 'M';
static int const LIBPQ_DIAG_MESSAGE_DETAIL = 'D';
static int const LIBPQ_DIAG_MESSAGE_HINT = 'H';
static int const LIBPQ_DIAG_STATEMENT_POSITION = 'P';
static int const LIBPQ_DIAG_INTERNAL_POSITION = 'p';
static int const LIBPQ_DIAG_INTERNAL_QUERY = 'q';
static int const LIBPQ_DIAG_CONTEXT = 'W';
static int const LIBPQ_DIAG_SCHEMA_NAME = 's';
static int const LIBPQ_DIAG_TABLE_NAME = 't';
static int const LIBPQ_DIAG_COLUMN_NAME = 'c';
static int const LIBPQ_DIAG_DATATYPE_NAME  = 'd';
static int const LIBPQ_DIAG_CONSTRAINT_NAME = 'n';
static int const LIBPQ_DIAG_SOURCE_FILE = 'F';
static int const LIBPQ_DIAG_SOURCE_LINE = 'L';
static int const LIBPQ_DIAG_SOURCE_FUNCTION = 'R';


static int const _PG_VERSION = 1115136;


/************************************************************/

static void *_cffi_types[] = {
/*  0 */ _CFFI_OP(_CFFI_OP_FUNCTION, 44), // PGcancel *()(PGconn *)
/*  1 */ _CFFI_OP(_CFFI_OP_POINTER, 212), // PGconn *
/*  2 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  3 */ _CFFI_OP(_CFFI_OP_FUNCTION, 1), // PGconn *()(char const *)
/*  4 */ _CFFI_OP(_CFFI_OP_POINTER, 217), // char const *
/*  5 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  6 */ _CFFI_OP(_CFFI_OP_FUNCTION, 213), // PGnotify *()(PGconn *)
/*  7 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/*  8 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/*  9 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // PGresult *()(PGconn *)
/* 10 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 11 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 12 */ _CFFI_OP(_CFFI_OP_FUNCTION, 25), // PGresult *()(PGconn *, char const *)
/* 13 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 14 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 15 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 16 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // char *()(PGconn *, char const *, size_t)
/* 17 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 18 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 19 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28), // size_t
/* 20 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 21 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // char *()(PGconn const *)
/* 22 */ _CFFI_OP(_CFFI_OP_POINTER, 212), // PGconn const *
/* 23 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 24 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // char *()(PGresult *)
/* 25 */ _CFFI_OP(_CFFI_OP_POINTER, 215), // PGresult *
/* 26 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 27 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // char *()(PGresult const *)
/* 28 */ _CFFI_OP(_CFFI_OP_POINTER, 215), // PGresult const *
/* 29 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 30 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // char *()(PGresult const *, int)
/* 31 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 32 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7), // int
/* 33 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 34 */ _CFFI_OP(_CFFI_OP_FUNCTION, 45), // char *()(PGresult const *, int, int)
/* 35 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 36 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 37 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 38 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 39 */ _CFFI_OP(_CFFI_OP_FUNCTION, 4), // char const *()(PGconn const *, char const *)
/* 40 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 41 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 42 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 43 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGcancel *, char *, int)
/* 44 */ _CFFI_OP(_CFFI_OP_POINTER, 211), // PGcancel *
/* 45 */ _CFFI_OP(_CFFI_OP_POINTER, 217), // char *
/* 46 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 47 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 48 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *)
/* 49 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 50 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 51 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, char * *, int)
/* 52 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 53 */ _CFFI_OP(_CFFI_OP_POINTER, 45), // char * *
/* 54 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 55 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 56 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, char const *)
/* 57 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 58 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 59 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 60 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, char const *, int)
/* 61 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 62 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 63 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 64 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 65 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, int)
/* 66 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 67 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 68 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 69 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, int, char *, size_t)
/* 70 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 71 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 72 */ _CFFI_OP(_CFFI_OP_NOOP, 45),
/* 73 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 74 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 75 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, int, char const *, size_t)
/* 76 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 77 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 78 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 79 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 80 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 81 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, int, int, int)
/* 82 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 83 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 84 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 85 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 86 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 87 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, int, size_t)
/* 88 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 89 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 90 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 91 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 92 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, unsigned int)
/* 93 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 94 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8), // unsigned int
/* 95 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 96 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, unsigned int, char const *)
/* 97 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 98 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 99 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 100 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 101 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn *, unsigned int, int)
/* 102 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 103 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 104 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 105 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 106 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGconn const *)
/* 107 */ _CFFI_OP(_CFFI_OP_NOOP, 22),
/* 108 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 109 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGresult const *)
/* 110 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 111 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 112 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGresult const *, int)
/* 113 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 114 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 115 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 116 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(PGresult const *, int, int)
/* 117 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 118 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 119 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 120 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 121 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(double *, PGresult const *, int, int)
/* 122 */ _CFFI_OP(_CFFI_OP_POINTER, 218), // double *
/* 123 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 124 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 125 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 126 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 127 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(float *, PGresult const *, int, int)
/* 128 */ _CFFI_OP(_CFFI_OP_POINTER, 219), // float *
/* 129 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 130 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 131 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 132 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 133 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(int32_t *, PGresult const *, int, int)
/* 134 */ _CFFI_OP(_CFFI_OP_POINTER, 220), // int32_t *
/* 135 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 136 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 137 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 138 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 139 */ _CFFI_OP(_CFFI_OP_FUNCTION, 32), // int()(int64_t *, PGresult const *, int, int)
/* 140 */ _CFFI_OP(_CFFI_OP_POINTER, 221), // int64_t *
/* 141 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 142 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 143 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 144 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 145 */ _CFFI_OP(_CFFI_OP_FUNCTION, 19), // size_t()(PGconn *, char *, char const *, size_t, int *)
/* 146 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 147 */ _CFFI_OP(_CFFI_OP_NOOP, 45),
/* 148 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 149 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 150 */ _CFFI_OP(_CFFI_OP_POINTER, 32), // int *
/* 151 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 152 */ _CFFI_OP(_CFFI_OP_FUNCTION, 19), // size_t()(char *, char const *, size_t)
/* 153 */ _CFFI_OP(_CFFI_OP_NOOP, 45),
/* 154 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 155 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 156 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 157 */ _CFFI_OP(_CFFI_OP_FUNCTION, 222), // unsigned char *()(PGconn *, unsigned char const *, size_t, size_t *)
/* 158 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 159 */ _CFFI_OP(_CFFI_OP_POINTER, 223), // unsigned char const *
/* 160 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 161 */ _CFFI_OP(_CFFI_OP_POINTER, 19), // size_t *
/* 162 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 163 */ _CFFI_OP(_CFFI_OP_FUNCTION, 222), // unsigned char *()(unsigned char const *, size_t *)
/* 164 */ _CFFI_OP(_CFFI_OP_NOOP, 159),
/* 165 */ _CFFI_OP(_CFFI_OP_NOOP, 161),
/* 166 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 167 */ _CFFI_OP(_CFFI_OP_FUNCTION, 222), // unsigned char *()(unsigned char const *, size_t, size_t *)
/* 168 */ _CFFI_OP(_CFFI_OP_NOOP, 159),
/* 169 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 28),
/* 170 */ _CFFI_OP(_CFFI_OP_NOOP, 161),
/* 171 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 172 */ _CFFI_OP(_CFFI_OP_FUNCTION, 94), // unsigned int()(PGconn *, char const *)
/* 173 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 174 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 175 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 176 */ _CFFI_OP(_CFFI_OP_FUNCTION, 94), // unsigned int()(PGconn *, unsigned int)
/* 177 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 178 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 8),
/* 179 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 180 */ _CFFI_OP(_CFFI_OP_FUNCTION, 94), // unsigned int()(PGresult const *)
/* 181 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 182 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 183 */ _CFFI_OP(_CFFI_OP_FUNCTION, 94), // unsigned int()(PGresult const *, int)
/* 184 */ _CFFI_OP(_CFFI_OP_NOOP, 28),
/* 185 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 7),
/* 186 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 187 */ _CFFI_OP(_CFFI_OP_FUNCTION, 224), // void()(PGcancel *)
/* 188 */ _CFFI_OP(_CFFI_OP_NOOP, 44),
/* 189 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 190 */ _CFFI_OP(_CFFI_OP_FUNCTION, 224), // void()(PGconn *)
/* 191 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 192 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 193 */ _CFFI_OP(_CFFI_OP_FUNCTION, 224), // void()(PGresult *)
/* 194 */ _CFFI_OP(_CFFI_OP_NOOP, 25),
/* 195 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 196 */ _CFFI_OP(_CFFI_OP_FUNCTION, 224), // void()(void *)
/* 197 */ _CFFI_OP(_CFFI_OP_POINTER, 224), // void *
/* 198 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 199 */ _CFFI_OP(_CFFI_OP_FUNCTION, 224), // void()(void *, char const *)
/* 200 */ _CFFI_OP(_CFFI_OP_NOOP, 197),
/* 201 */ _CFFI_OP(_CFFI_OP_NOOP, 4),
/* 202 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 203 */ _CFFI_OP(_CFFI_OP_FUNCTION, 205), // void(*()(PGconn *, void(*)(void *, char const *), void *))(void *, char const *)
/* 204 */ _CFFI_OP(_CFFI_OP_NOOP, 1),
/* 205 */ _CFFI_OP(_CFFI_OP_POINTER, 199), // void(*)(void *, char const *)
/* 206 */ _CFFI_OP(_CFFI_OP_NOOP, 197),
/* 207 */ _CFFI_OP(_CFFI_OP_FUNCTION_END, 0),
/* 208 */ _CFFI_OP(_CFFI_OP_ENUM, 0), // ConnStatusType
/* 209 */ _CFFI_OP(_CFFI_OP_ENUM, 1), // ExecStatusType
/* 210 */ _CFFI_OP(_CFFI_OP_ENUM, 2), // PGTransactionStatusType
/* 211 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 0), // PGcancel
/* 212 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 1), // PGconn
/* 213 */ _CFFI_OP(_CFFI_OP_POINTER, 214), // PGnotify *
/* 214 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 3), // PGnotify
/* 215 */ _CFFI_OP(_CFFI_OP_STRUCT_UNION, 2), // PGresult
/* 216 */ _CFFI_OP(_CFFI_OP_ENUM, 3), // PostgresPollingStatusType
/* 217 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 2), // char
/* 218 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 14), // double
/* 219 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 13), // float
/* 220 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 21), // int32_t
/* 221 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 23), // int64_t
/* 222 */ _CFFI_OP(_CFFI_OP_POINTER, 223), // unsigned char *
/* 223 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 4), // unsigned char
/* 224 */ _CFFI_OP(_CFFI_OP_PRIMITIVE, 0), // void
};

static int _cffi_const_CONNECTION_OK(unsigned long long *o)
{
  int n = (CONNECTION_OK) <= 0;
  *o = (unsigned long long)((CONNECTION_OK) | 0);  /* check that CONNECTION_OK is an integer */
  return n;
}

static int _cffi_const_CONNECTION_BAD(unsigned long long *o)
{
  int n = (CONNECTION_BAD) <= 0;
  *o = (unsigned long long)((CONNECTION_BAD) | 0);  /* check that CONNECTION_BAD is an integer */
  return n;
}

static int _cffi_const_CONNECTION_STARTED(unsigned long long *o)
{
  int n = (CONNECTION_STARTED) <= 0;
  *o = (unsigned long long)((CONNECTION_STARTED) | 0);  /* check that CONNECTION_STARTED is an integer */
  return n;
}

static int _cffi_const_CONNECTION_MADE(unsigned long long *o)
{
  int n = (CONNECTION_MADE) <= 0;
  *o = (unsigned long long)((CONNECTION_MADE) | 0);  /* check that CONNECTION_MADE is an integer */
  return n;
}

static int _cffi_const_CONNECTION_AWAITING_RESPONSE(unsigned long long *o)
{
  int n = (CONNECTION_AWAITING_RESPONSE) <= 0;
  *o = (unsigned long long)((CONNECTION_AWAITING_RESPONSE) | 0);  /* check that CONNECTION_AWAITING_RESPONSE is an integer */
  return n;
}

static int _cffi_const_CONNECTION_AUTH_OK(unsigned long long *o)
{
  int n = (CONNECTION_AUTH_OK) <= 0;
  *o = (unsigned long long)((CONNECTION_AUTH_OK) | 0);  /* check that CONNECTION_AUTH_OK is an integer */
  return n;
}

static int _cffi_const_CONNECTION_SETENV(unsigned long long *o)
{
  int n = (CONNECTION_SETENV) <= 0;
  *o = (unsigned long long)((CONNECTION_SETENV) | 0);  /* check that CONNECTION_SETENV is an integer */
  return n;
}

static int _cffi_const_CONNECTION_SSL_STARTUP(unsigned long long *o)
{
  int n = (CONNECTION_SSL_STARTUP) <= 0;
  *o = (unsigned long long)((CONNECTION_SSL_STARTUP) | 0);  /* check that CONNECTION_SSL_STARTUP is an integer */
  return n;
}

static int _cffi_const_CONNECTION_NEEDED(unsigned long long *o)
{
  int n = (CONNECTION_NEEDED) <= 0;
  *o = (unsigned long long)((CONNECTION_NEEDED) | 0);  /* check that CONNECTION_NEEDED is an integer */
  return n;
}

static int _cffi_const_PGRES_EMPTY_QUERY(unsigned long long *o)
{
  int n = (PGRES_EMPTY_QUERY) <= 0;
  *o = (unsigned long long)((PGRES_EMPTY_QUERY) | 0);  /* check that PGRES_EMPTY_QUERY is an integer */
  return n;
}

static int _cffi_const_PGRES_COMMAND_OK(unsigned long long *o)
{
  int n = (PGRES_COMMAND_OK) <= 0;
  *o = (unsigned long long)((PGRES_COMMAND_OK) | 0);  /* check that PGRES_COMMAND_OK is an integer */
  return n;
}

static int _cffi_const_PGRES_TUPLES_OK(unsigned long long *o)
{
  int n = (PGRES_TUPLES_OK) <= 0;
  *o = (unsigned long long)((PGRES_TUPLES_OK) | 0);  /* check that PGRES_TUPLES_OK is an integer */
  return n;
}

static int _cffi_const_PGRES_COPY_OUT(unsigned long long *o)
{
  int n = (PGRES_COPY_OUT) <= 0;
  *o = (unsigned long long)((PGRES_COPY_OUT) | 0);  /* check that PGRES_COPY_OUT is an integer */
  return n;
}

static int _cffi_const_PGRES_COPY_IN(unsigned long long *o)
{
  int n = (PGRES_COPY_IN) <= 0;
  *o = (unsigned long long)((PGRES_COPY_IN) | 0);  /* check that PGRES_COPY_IN is an integer */
  return n;
}

static int _cffi_const_PGRES_BAD_RESPONSE(unsigned long long *o)
{
  int n = (PGRES_BAD_RESPONSE) <= 0;
  *o = (unsigned long long)((PGRES_BAD_RESPONSE) | 0);  /* check that PGRES_BAD_RESPONSE is an integer */
  return n;
}

static int _cffi_const_PGRES_NONFATAL_ERROR(unsigned long long *o)
{
  int n = (PGRES_NONFATAL_ERROR) <= 0;
  *o = (unsigned long long)((PGRES_NONFATAL_ERROR) | 0);  /* check that PGRES_NONFATAL_ERROR is an integer */
  return n;
}

static int _cffi_const_PGRES_FATAL_ERROR(unsigned long long *o)
{
  int n = (PGRES_FATAL_ERROR) <= 0;
  *o = (unsigned long long)((PGRES_FATAL_ERROR) | 0);  /* check that PGRES_FATAL_ERROR is an integer */
  return n;
}

static int _cffi_const_PQTRANS_IDLE(unsigned long long *o)
{
  int n = (PQTRANS_IDLE) <= 0;
  *o = (unsigned long long)((PQTRANS_IDLE) | 0);  /* check that PQTRANS_IDLE is an integer */
  return n;
}

static int _cffi_const_PQTRANS_ACTIVE(unsigned long long *o)
{
  int n = (PQTRANS_ACTIVE) <= 0;
  *o = (unsigned long long)((PQTRANS_ACTIVE) | 0);  /* check that PQTRANS_ACTIVE is an integer */
  return n;
}

static int _cffi_const_PQTRANS_INTRANS(unsigned long long *o)
{
  int n = (PQTRANS_INTRANS) <= 0;
  *o = (unsigned long long)((PQTRANS_INTRANS) | 0);  /* check that PQTRANS_INTRANS is an integer */
  return n;
}

static int _cffi_const_PQTRANS_INERROR(unsigned long long *o)
{
  int n = (PQTRANS_INERROR) <= 0;
  *o = (unsigned long long)((PQTRANS_INERROR) | 0);  /* check that PQTRANS_INERROR is an integer */
  return n;
}

static int _cffi_const_PQTRANS_UNKNOWN(unsigned long long *o)
{
  int n = (PQTRANS_UNKNOWN) <= 0;
  *o = (unsigned long long)((PQTRANS_UNKNOWN) | 0);  /* check that PQTRANS_UNKNOWN is an integer */
  return n;
}

static int _cffi_const_PGRES_POLLING_FAILED(unsigned long long *o)
{
  int n = (PGRES_POLLING_FAILED) <= 0;
  *o = (unsigned long long)((PGRES_POLLING_FAILED) | 0);  /* check that PGRES_POLLING_FAILED is an integer */
  return n;
}

static int _cffi_const_PGRES_POLLING_READING(unsigned long long *o)
{
  int n = (PGRES_POLLING_READING) <= 0;
  *o = (unsigned long long)((PGRES_POLLING_READING) | 0);  /* check that PGRES_POLLING_READING is an integer */
  return n;
}

static int _cffi_const_PGRES_POLLING_WRITING(unsigned long long *o)
{
  int n = (PGRES_POLLING_WRITING) <= 0;
  *o = (unsigned long long)((PGRES_POLLING_WRITING) | 0);  /* check that PGRES_POLLING_WRITING is an integer */
  return n;
}

static int _cffi_const_PGRES_POLLING_OK(unsigned long long *o)
{
  int n = (PGRES_POLLING_OK) <= 0;
  *o = (unsigned long long)((PGRES_POLLING_OK) | 0);  /* check that PGRES_POLLING_OK is an integer */
  return n;
}

static int _cffi_const_PGRES_POLLING_ACTIVE(unsigned long long *o)
{
  int n = (PGRES_POLLING_ACTIVE) <= 0;
  *o = (unsigned long long)((PGRES_POLLING_ACTIVE) | 0);  /* check that PGRES_POLLING_ACTIVE is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_COLUMN_NAME(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_COLUMN_NAME) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_COLUMN_NAME) | 0);  /* check that LIBPQ_DIAG_COLUMN_NAME is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_CONSTRAINT_NAME(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_CONSTRAINT_NAME) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_CONSTRAINT_NAME) | 0);  /* check that LIBPQ_DIAG_CONSTRAINT_NAME is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_CONTEXT(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_CONTEXT) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_CONTEXT) | 0);  /* check that LIBPQ_DIAG_CONTEXT is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_DATATYPE_NAME(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_DATATYPE_NAME) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_DATATYPE_NAME) | 0);  /* check that LIBPQ_DIAG_DATATYPE_NAME is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_INTERNAL_POSITION(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_INTERNAL_POSITION) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_INTERNAL_POSITION) | 0);  /* check that LIBPQ_DIAG_INTERNAL_POSITION is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_INTERNAL_QUERY(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_INTERNAL_QUERY) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_INTERNAL_QUERY) | 0);  /* check that LIBPQ_DIAG_INTERNAL_QUERY is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_MESSAGE_DETAIL(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_MESSAGE_DETAIL) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_MESSAGE_DETAIL) | 0);  /* check that LIBPQ_DIAG_MESSAGE_DETAIL is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_MESSAGE_HINT(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_MESSAGE_HINT) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_MESSAGE_HINT) | 0);  /* check that LIBPQ_DIAG_MESSAGE_HINT is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_MESSAGE_PRIMARY(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_MESSAGE_PRIMARY) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_MESSAGE_PRIMARY) | 0);  /* check that LIBPQ_DIAG_MESSAGE_PRIMARY is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_SCHEMA_NAME(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_SCHEMA_NAME) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_SCHEMA_NAME) | 0);  /* check that LIBPQ_DIAG_SCHEMA_NAME is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_SEVERITY(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_SEVERITY) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_SEVERITY) | 0);  /* check that LIBPQ_DIAG_SEVERITY is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_SOURCE_FILE(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_SOURCE_FILE) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_SOURCE_FILE) | 0);  /* check that LIBPQ_DIAG_SOURCE_FILE is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_SOURCE_FUNCTION(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_SOURCE_FUNCTION) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_SOURCE_FUNCTION) | 0);  /* check that LIBPQ_DIAG_SOURCE_FUNCTION is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_SOURCE_LINE(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_SOURCE_LINE) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_SOURCE_LINE) | 0);  /* check that LIBPQ_DIAG_SOURCE_LINE is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_SQLSTATE(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_SQLSTATE) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_SQLSTATE) | 0);  /* check that LIBPQ_DIAG_SQLSTATE is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_STATEMENT_POSITION(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_STATEMENT_POSITION) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_STATEMENT_POSITION) | 0);  /* check that LIBPQ_DIAG_STATEMENT_POSITION is an integer */
  return n;
}

static int _cffi_const_LIBPQ_DIAG_TABLE_NAME(unsigned long long *o)
{
  int n = (LIBPQ_DIAG_TABLE_NAME) <= 0;
  *o = (unsigned long long)((LIBPQ_DIAG_TABLE_NAME) | 0);  /* check that LIBPQ_DIAG_TABLE_NAME is an integer */
  return n;
}

static int _cffi_const__PG_VERSION(unsigned long long *o)
{
  int n = (_PG_VERSION) <= 0;
  *o = (unsigned long long)((_PG_VERSION) | 0);  /* check that _PG_VERSION is an integer */
  return n;
}

static int _cffi_d_PQEgetdouble(double * x0, PGresult const * x1, int x2, int x3)
{
  return PQEgetdouble(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQEgetdouble(PyObject *self, PyObject *args)
{
  double * x0;
  PGresult const * x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "PQEgetdouble", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(122), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (double *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(122), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQEgetdouble(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQEgetdouble _cffi_d_PQEgetdouble
#endif

static int _cffi_d_PQEgetfloat(float * x0, PGresult const * x1, int x2, int x3)
{
  return PQEgetfloat(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQEgetfloat(PyObject *self, PyObject *args)
{
  float * x0;
  PGresult const * x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "PQEgetfloat", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(128), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (float *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(128), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQEgetfloat(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQEgetfloat _cffi_d_PQEgetfloat
#endif

static int _cffi_d_PQEgetint(int32_t * x0, PGresult const * x1, int x2, int x3)
{
  return PQEgetint(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQEgetint(PyObject *self, PyObject *args)
{
  int32_t * x0;
  PGresult const * x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "PQEgetint", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(134), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int32_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(134), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQEgetint(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQEgetint _cffi_d_PQEgetint
#endif

static int _cffi_d_PQEgetlong(int64_t * x0, PGresult const * x1, int x2, int x3)
{
  return PQEgetlong(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQEgetlong(PyObject *self, PyObject *args)
{
  int64_t * x0;
  PGresult const * x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "PQEgetlong", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(140), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (int64_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(140), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQEgetlong(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQEgetlong _cffi_d_PQEgetlong
#endif

static int _cffi_d_PQbackendPID(PGconn const * x0)
{
  return PQbackendPID(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQbackendPID(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQbackendPID(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQbackendPID _cffi_d_PQbackendPID
#endif

static int _cffi_d_PQcancel(PGcancel * x0, char * x1, int x2)
{
  return PQcancel(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQcancel(PyObject *self, PyObject *args)
{
  PGcancel * x0;
  char * x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQcancel", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(44), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGcancel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(44), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQcancel(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQcancel _cffi_d_PQcancel
#endif

static void _cffi_d_PQclear(PGresult * x0)
{
  PQclear(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQclear(PyObject *self, PyObject *arg0)
{
  PGresult * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PQclear(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_PQclear _cffi_d_PQclear
#endif

static char * _cffi_d_PQcmdStatus(PGresult * x0)
{
  return PQcmdStatus(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQcmdStatus(PyObject *self, PyObject *arg0)
{
  PGresult * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQcmdStatus(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQcmdStatus _cffi_d_PQcmdStatus
#endif

static char * _cffi_d_PQcmdTuples(PGresult * x0)
{
  return PQcmdTuples(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQcmdTuples(PyObject *self, PyObject *arg0)
{
  PGresult * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(25), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(25), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQcmdTuples(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQcmdTuples _cffi_d_PQcmdTuples
#endif

static int _cffi_d_PQconnectPoll(PGconn * x0)
{
  return PQconnectPoll(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQconnectPoll(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQconnectPoll(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQconnectPoll _cffi_d_PQconnectPoll
#endif

static PGconn * _cffi_d_PQconnectStart(char const * x0)
{
  return PQconnectStart(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQconnectStart(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PGconn * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQconnectStart(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQconnectStart _cffi_d_PQconnectStart
#endif

static PGconn * _cffi_d_PQconnectdb(char const * x0)
{
  return PQconnectdb(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQconnectdb(PyObject *self, PyObject *arg0)
{
  char const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PGconn * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQconnectdb(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(1));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQconnectdb _cffi_d_PQconnectdb
#endif

static int _cffi_d_PQconsumeInput(PGconn * x0)
{
  return PQconsumeInput(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQconsumeInput(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQconsumeInput(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQconsumeInput _cffi_d_PQconsumeInput
#endif

static char * _cffi_d_PQerrorMessage(PGconn const * x0)
{
  return PQerrorMessage(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQerrorMessage(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQerrorMessage(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQerrorMessage _cffi_d_PQerrorMessage
#endif

static unsigned char * _cffi_d_PQescapeBytea(unsigned char const * x0, size_t x1, size_t * x2)
{
  return PQescapeBytea(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQescapeBytea(PyObject *self, PyObject *args)
{
  unsigned char const * x0;
  size_t x1;
  size_t * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQescapeBytea", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(159), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(159), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, size_t);
  if (x1 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(161), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(161), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQescapeBytea(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(222));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQescapeBytea _cffi_d_PQescapeBytea
#endif

static unsigned char * _cffi_d_PQescapeByteaConn(PGconn * x0, unsigned char const * x1, size_t x2, size_t * x3)
{
  return PQescapeByteaConn(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQescapeByteaConn(PyObject *self, PyObject *args)
{
  PGconn * x0;
  unsigned char const * x1;
  size_t x2;
  size_t * x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "PQescapeByteaConn", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(159), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(159), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(161), arg3, (char **)&x3);
  if (datasize != 0) {
    x3 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(161), arg3, (char **)&x3,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQescapeByteaConn(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(222));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQescapeByteaConn _cffi_d_PQescapeByteaConn
#endif

static char * _cffi_d_PQescapeLiteral(PGconn * x0, char const * x1, size_t x2)
{
  return PQescapeLiteral(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQescapeLiteral(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char const * x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQescapeLiteral", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQescapeLiteral(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQescapeLiteral _cffi_d_PQescapeLiteral
#endif

static size_t _cffi_d_PQescapeString(char * x0, char const * x1, size_t x2)
{
  return PQescapeString(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQescapeString(PyObject *self, PyObject *args)
{
  char * x0;
  char const * x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  size_t result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQescapeString", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQescapeString(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, size_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQescapeString _cffi_d_PQescapeString
#endif

static size_t _cffi_d_PQescapeStringConn(PGconn * x0, char * x1, char const * x2, size_t x3, int * x4)
{
  return PQescapeStringConn(x0, x1, x2, x3, x4);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQescapeStringConn(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char * x1;
  char const * x2;
  size_t x3;
  int * x4;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  size_t result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;
  PyObject *arg4;

  if (!PyArg_UnpackTuple(args, "PQescapeStringConn", 5, 5, &arg0, &arg1, &arg2, &arg3, &arg4))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(150), arg4, (char **)&x4);
  if (datasize != 0) {
    x4 = ((size_t)datasize) <= 640 ? (int *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(150), arg4, (char **)&x4,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQescapeStringConn(x0, x1, x2, x3, x4); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, size_t);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQescapeStringConn _cffi_d_PQescapeStringConn
#endif

static PGresult * _cffi_d_PQexec(PGconn * x0, char const * x1)
{
  return PQexec(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQexec(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PGresult * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQexec", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQexec(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(25));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQexec _cffi_d_PQexec
#endif

static void _cffi_d_PQfinish(PGconn * x0)
{
  PQfinish(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQfinish(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PQfinish(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_PQfinish _cffi_d_PQfinish
#endif

static int _cffi_d_PQflush(PGconn * x0)
{
  return PQflush(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQflush(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQflush(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQflush _cffi_d_PQflush
#endif

static int _cffi_d_PQfmod(PGresult const * x0, int x1)
{
  return PQfmod(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQfmod(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQfmod", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQfmod(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQfmod _cffi_d_PQfmod
#endif

static char * _cffi_d_PQfname(PGresult const * x0, int x1)
{
  return PQfname(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQfname(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQfname", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQfname(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQfname _cffi_d_PQfname
#endif

static void _cffi_d_PQfreeCancel(PGcancel * x0)
{
  PQfreeCancel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQfreeCancel(PyObject *self, PyObject *arg0)
{
  PGcancel * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(44), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGcancel *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(44), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PQfreeCancel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_PQfreeCancel _cffi_d_PQfreeCancel
#endif

static void _cffi_d_PQfreemem(void * x0)
{
  PQfreemem(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQfreemem(PyObject *self, PyObject *arg0)
{
  void * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(197), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(197), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { PQfreemem(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  Py_INCREF(Py_None);
  return Py_None;
}
#else
#  define _cffi_f_PQfreemem _cffi_d_PQfreemem
#endif

static int _cffi_d_PQfsize(PGresult const * x0, int x1)
{
  return PQfsize(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQfsize(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQfsize", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQfsize(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQfsize _cffi_d_PQfsize
#endif

static unsigned int _cffi_d_PQftype(PGresult const * x0, int x1)
{
  return PQftype(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQftype(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQftype", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQftype(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQftype _cffi_d_PQftype
#endif

static PGcancel * _cffi_d_PQgetCancel(PGconn * x0)
{
  return PQgetCancel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQgetCancel(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PGcancel * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQgetCancel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(44));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQgetCancel _cffi_d_PQgetCancel
#endif

static int _cffi_d_PQgetCopyData(PGconn * x0, char * * x1, int x2)
{
  return PQgetCopyData(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQgetCopyData(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char * * x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQgetCopyData", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(53), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char * *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(53), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQgetCopyData(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQgetCopyData _cffi_d_PQgetCopyData
#endif

static PGresult * _cffi_d_PQgetResult(PGconn * x0)
{
  return PQgetResult(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQgetResult(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PGresult * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQgetResult(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(25));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQgetResult _cffi_d_PQgetResult
#endif

static int _cffi_d_PQgetisnull(PGresult const * x0, int x1, int x2)
{
  return PQgetisnull(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQgetisnull(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQgetisnull", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQgetisnull(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQgetisnull _cffi_d_PQgetisnull
#endif

static int _cffi_d_PQgetlength(PGresult const * x0, int x1, int x2)
{
  return PQgetlength(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQgetlength(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQgetlength", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQgetlength(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQgetlength _cffi_d_PQgetlength
#endif

static char * _cffi_d_PQgetvalue(PGresult const * x0, int x1, int x2)
{
  return PQgetvalue(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQgetvalue(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQgetvalue", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQgetvalue(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQgetvalue _cffi_d_PQgetvalue
#endif

static int _cffi_d_PQisBusy(PGconn * x0)
{
  return PQisBusy(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQisBusy(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQisBusy(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQisBusy _cffi_d_PQisBusy
#endif

static int _cffi_d_PQnfields(PGresult const * x0)
{
  return PQnfields(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQnfields(PyObject *self, PyObject *arg0)
{
  PGresult const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQnfields(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQnfields _cffi_d_PQnfields
#endif

static PGnotify * _cffi_d_PQnotifies(PGconn * x0)
{
  return PQnotifies(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQnotifies(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  PGnotify * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQnotifies(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(213));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQnotifies _cffi_d_PQnotifies
#endif

static int _cffi_d_PQntuples(PGresult const * x0)
{
  return PQntuples(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQntuples(PyObject *self, PyObject *arg0)
{
  PGresult const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQntuples(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQntuples _cffi_d_PQntuples
#endif

static unsigned int _cffi_d_PQoidValue(PGresult const * x0)
{
  return PQoidValue(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQoidValue(PyObject *self, PyObject *arg0)
{
  PGresult const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQoidValue(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQoidValue _cffi_d_PQoidValue
#endif

static char const * _cffi_d_PQparameterStatus(PGconn const * x0, char const * x1)
{
  return PQparameterStatus(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQparameterStatus(PyObject *self, PyObject *args)
{
  PGconn const * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char const * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQparameterStatus", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQparameterStatus(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(4));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQparameterStatus _cffi_d_PQparameterStatus
#endif

static int _cffi_d_PQprotocolVersion(PGconn const * x0)
{
  return PQprotocolVersion(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQprotocolVersion(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQprotocolVersion(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQprotocolVersion _cffi_d_PQprotocolVersion
#endif

static int _cffi_d_PQputCopyData(PGconn * x0, char const * x1, int x2)
{
  return PQputCopyData(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQputCopyData(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char const * x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQputCopyData", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQputCopyData(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQputCopyData _cffi_d_PQputCopyData
#endif

static int _cffi_d_PQputCopyEnd(PGconn * x0, char const * x1)
{
  return PQputCopyEnd(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQputCopyEnd(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQputCopyEnd", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQputCopyEnd(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQputCopyEnd _cffi_d_PQputCopyEnd
#endif

static int _cffi_d_PQrequestCancel(PGconn * x0)
{
  return PQrequestCancel(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQrequestCancel(PyObject *self, PyObject *arg0)
{
  PGconn * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQrequestCancel(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQrequestCancel _cffi_d_PQrequestCancel
#endif

static char * _cffi_d_PQresultErrorField(PGresult const * x0, int x1)
{
  return PQresultErrorField(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQresultErrorField(PyObject *self, PyObject *args)
{
  PGresult const * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQresultErrorField", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQresultErrorField(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQresultErrorField _cffi_d_PQresultErrorField
#endif

static char * _cffi_d_PQresultErrorMessage(PGresult const * x0)
{
  return PQresultErrorMessage(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQresultErrorMessage(PyObject *self, PyObject *arg0)
{
  PGresult const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  char * result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQresultErrorMessage(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(45));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQresultErrorMessage _cffi_d_PQresultErrorMessage
#endif

static int _cffi_d_PQresultStatus(PGresult const * x0)
{
  return PQresultStatus(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQresultStatus(PyObject *self, PyObject *arg0)
{
  PGresult const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(28), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGresult const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(28), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQresultStatus(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQresultStatus _cffi_d_PQresultStatus
#endif

static int _cffi_d_PQsendQuery(PGconn * x0, char const * x1)
{
  return PQsendQuery(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQsendQuery(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQsendQuery", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQsendQuery(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQsendQuery _cffi_d_PQsendQuery
#endif

static int _cffi_d_PQserverVersion(PGconn const * x0)
{
  return PQserverVersion(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQserverVersion(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQserverVersion(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQserverVersion _cffi_d_PQserverVersion
#endif

static void(* _cffi_d_PQsetNoticeProcessor(PGconn * x0, void(* x1)(void *, char const *), void * x2))(void *, char const *)
{
  return PQsetNoticeProcessor(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQsetNoticeProcessor(PyObject *self, PyObject *args)
{
  PGconn * x0;
  void(* x1)(void *, char const *);
  void * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  void(* result)(void *, char const *);
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "PQsetNoticeProcessor", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = (void(*)(void *, char const *))_cffi_to_c_pointer(arg1, _cffi_type(205));
  if (x1 == (void(*)(void *, char const *))NULL && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(197), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (void *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(197), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQsetNoticeProcessor(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(205));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQsetNoticeProcessor _cffi_d_PQsetNoticeProcessor
#endif

static int _cffi_d_PQsetnonblocking(PGconn * x0, int x1)
{
  return PQsetnonblocking(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQsetnonblocking(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQsetnonblocking", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQsetnonblocking(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQsetnonblocking _cffi_d_PQsetnonblocking
#endif

static int _cffi_d_PQsocket(PGconn const * x0)
{
  return PQsocket(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQsocket(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQsocket(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQsocket _cffi_d_PQsocket
#endif

static int _cffi_d_PQstatus(PGconn const * x0)
{
  return PQstatus(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQstatus(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQstatus(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQstatus _cffi_d_PQstatus
#endif

static int _cffi_d_PQtransactionStatus(PGconn const * x0)
{
  return PQtransactionStatus(x0);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQtransactionStatus(PyObject *self, PyObject *arg0)
{
  PGconn const * x0;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(22), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(22), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQtransactionStatus(x0); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQtransactionStatus _cffi_d_PQtransactionStatus
#endif

static unsigned char * _cffi_d_PQunescapeBytea(unsigned char const * x0, size_t * x1)
{
  return PQunescapeBytea(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_PQunescapeBytea(PyObject *self, PyObject *args)
{
  unsigned char const * x0;
  size_t * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned char * result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "PQunescapeBytea", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(159), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (unsigned char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(159), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(161), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (size_t *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(161), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = PQunescapeBytea(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_pointer((char *)result, _cffi_type(222));
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_PQunescapeBytea _cffi_d_PQunescapeBytea
#endif

static int _cffi_d_lo_close(PGconn * x0, int x1)
{
  return lo_close(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_close(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "lo_close", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_close(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_close _cffi_d_lo_close
#endif

static unsigned int _cffi_d_lo_create(PGconn * x0, unsigned int x1)
{
  return lo_create(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_create(PyObject *self, PyObject *args)
{
  PGconn * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "lo_create", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_create(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_create _cffi_d_lo_create
#endif

static int _cffi_d_lo_export(PGconn * x0, unsigned int x1, char const * x2)
{
  return lo_export(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_export(PyObject *self, PyObject *args)
{
  PGconn * x0;
  unsigned int x1;
  char const * x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "lo_export", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_export(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_export _cffi_d_lo_export
#endif

static unsigned int _cffi_d_lo_import(PGconn * x0, char const * x1)
{
  return lo_import(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_import(PyObject *self, PyObject *args)
{
  PGconn * x0;
  char const * x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  unsigned int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "lo_import", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg1, (char **)&x1);
  if (datasize != 0) {
    x1 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg1, (char **)&x1,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_import(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, unsigned int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_import _cffi_d_lo_import
#endif

static int _cffi_d_lo_lseek(PGconn * x0, int x1, int x2, int x3)
{
  return lo_lseek(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_lseek(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  int x2;
  int x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "lo_lseek", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  x3 = _cffi_to_c_int(arg3, int);
  if (x3 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_lseek(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_lseek _cffi_d_lo_lseek
#endif

static int _cffi_d_lo_open(PGconn * x0, unsigned int x1, int x2)
{
  return lo_open(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_open(PyObject *self, PyObject *args)
{
  PGconn * x0;
  unsigned int x1;
  int x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "lo_open", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, int);
  if (x2 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_open(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_open _cffi_d_lo_open
#endif

static int _cffi_d_lo_read(PGconn * x0, int x1, char * x2, size_t x3)
{
  return lo_read(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_read(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  char * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "lo_read", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(45), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(45), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_read(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_read _cffi_d_lo_read
#endif

static int _cffi_d_lo_tell(PGconn * x0, int x1)
{
  return lo_tell(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_tell(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "lo_tell", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_tell(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_tell _cffi_d_lo_tell
#endif

static int _cffi_d_lo_truncate(PGconn * x0, int x1, size_t x2)
{
  return lo_truncate(x0, x1, x2);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_truncate(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  size_t x2;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;

  if (!PyArg_UnpackTuple(args, "lo_truncate", 3, 3, &arg0, &arg1, &arg2))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  x2 = _cffi_to_c_int(arg2, size_t);
  if (x2 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_truncate(x0, x1, x2); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_truncate _cffi_d_lo_truncate
#endif

static int _cffi_d_lo_unlink(PGconn * x0, unsigned int x1)
{
  return lo_unlink(x0, x1);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_unlink(PyObject *self, PyObject *args)
{
  PGconn * x0;
  unsigned int x1;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;

  if (!PyArg_UnpackTuple(args, "lo_unlink", 2, 2, &arg0, &arg1))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, unsigned int);
  if (x1 == (unsigned int)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_unlink(x0, x1); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_unlink _cffi_d_lo_unlink
#endif

static int _cffi_d_lo_write(PGconn * x0, int x1, char const * x2, size_t x3)
{
  return lo_write(x0, x1, x2, x3);
}
#ifndef PYPY_VERSION
static PyObject *
_cffi_f_lo_write(PyObject *self, PyObject *args)
{
  PGconn * x0;
  int x1;
  char const * x2;
  size_t x3;
  Py_ssize_t datasize;
  struct _cffi_freeme_s *large_args_free = NULL;
  int result;
  PyObject *pyresult;
  PyObject *arg0;
  PyObject *arg1;
  PyObject *arg2;
  PyObject *arg3;

  if (!PyArg_UnpackTuple(args, "lo_write", 4, 4, &arg0, &arg1, &arg2, &arg3))
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(1), arg0, (char **)&x0);
  if (datasize != 0) {
    x0 = ((size_t)datasize) <= 640 ? (PGconn *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(1), arg0, (char **)&x0,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x1 = _cffi_to_c_int(arg1, int);
  if (x1 == (int)-1 && PyErr_Occurred())
    return NULL;

  datasize = _cffi_prepare_pointer_call_argument(
      _cffi_type(4), arg2, (char **)&x2);
  if (datasize != 0) {
    x2 = ((size_t)datasize) <= 640 ? (char const *)alloca((size_t)datasize) : NULL;
    if (_cffi_convert_array_argument(_cffi_type(4), arg2, (char **)&x2,
            datasize, &large_args_free) < 0)
      return NULL;
  }

  x3 = _cffi_to_c_int(arg3, size_t);
  if (x3 == (size_t)-1 && PyErr_Occurred())
    return NULL;

  Py_BEGIN_ALLOW_THREADS
  _cffi_restore_errno();
  { result = lo_write(x0, x1, x2, x3); }
  _cffi_save_errno();
  Py_END_ALLOW_THREADS

  (void)self; /* unused */
  pyresult = _cffi_from_c_int(result, int);
  if (large_args_free != NULL) _cffi_free_array_arguments(large_args_free);
  return pyresult;
}
#else
#  define _cffi_f_lo_write _cffi_d_lo_write
#endif

_CFFI_UNUSED_FN
static void _cffi_checkfld__PGnotify(PGnotify *p)
{
  /* only to generate compile-time warnings or errors */
  (void)p;
  { char * *tmp = &p->relname; (void)tmp; }
  (void)((p->be_pid) | 0);  /* check that 'PGnotify.be_pid' is an integer */
  { char * *tmp = &p->extra; (void)tmp; }
}
struct _cffi_align__PGnotify { char x; PGnotify y; };

static const struct _cffi_global_s _cffi_globals[] = {
  { "CONNECTION_AUTH_OK", (void *)_cffi_const_CONNECTION_AUTH_OK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_AWAITING_RESPONSE", (void *)_cffi_const_CONNECTION_AWAITING_RESPONSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_BAD", (void *)_cffi_const_CONNECTION_BAD, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_MADE", (void *)_cffi_const_CONNECTION_MADE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_NEEDED", (void *)_cffi_const_CONNECTION_NEEDED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_OK", (void *)_cffi_const_CONNECTION_OK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_SETENV", (void *)_cffi_const_CONNECTION_SETENV, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_SSL_STARTUP", (void *)_cffi_const_CONNECTION_SSL_STARTUP, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "CONNECTION_STARTED", (void *)_cffi_const_CONNECTION_STARTED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "LIBPQ_DIAG_COLUMN_NAME", (void *)_cffi_const_LIBPQ_DIAG_COLUMN_NAME, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_CONSTRAINT_NAME", (void *)_cffi_const_LIBPQ_DIAG_CONSTRAINT_NAME, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_CONTEXT", (void *)_cffi_const_LIBPQ_DIAG_CONTEXT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_DATATYPE_NAME", (void *)_cffi_const_LIBPQ_DIAG_DATATYPE_NAME, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_INTERNAL_POSITION", (void *)_cffi_const_LIBPQ_DIAG_INTERNAL_POSITION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_INTERNAL_QUERY", (void *)_cffi_const_LIBPQ_DIAG_INTERNAL_QUERY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_MESSAGE_DETAIL", (void *)_cffi_const_LIBPQ_DIAG_MESSAGE_DETAIL, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_MESSAGE_HINT", (void *)_cffi_const_LIBPQ_DIAG_MESSAGE_HINT, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_MESSAGE_PRIMARY", (void *)_cffi_const_LIBPQ_DIAG_MESSAGE_PRIMARY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_SCHEMA_NAME", (void *)_cffi_const_LIBPQ_DIAG_SCHEMA_NAME, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_SEVERITY", (void *)_cffi_const_LIBPQ_DIAG_SEVERITY, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_SOURCE_FILE", (void *)_cffi_const_LIBPQ_DIAG_SOURCE_FILE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_SOURCE_FUNCTION", (void *)_cffi_const_LIBPQ_DIAG_SOURCE_FUNCTION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_SOURCE_LINE", (void *)_cffi_const_LIBPQ_DIAG_SOURCE_LINE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_SQLSTATE", (void *)_cffi_const_LIBPQ_DIAG_SQLSTATE, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_STATEMENT_POSITION", (void *)_cffi_const_LIBPQ_DIAG_STATEMENT_POSITION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "LIBPQ_DIAG_TABLE_NAME", (void *)_cffi_const_LIBPQ_DIAG_TABLE_NAME, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "PGRES_BAD_RESPONSE", (void *)_cffi_const_PGRES_BAD_RESPONSE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_COMMAND_OK", (void *)_cffi_const_PGRES_COMMAND_OK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_COPY_IN", (void *)_cffi_const_PGRES_COPY_IN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_COPY_OUT", (void *)_cffi_const_PGRES_COPY_OUT, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_EMPTY_QUERY", (void *)_cffi_const_PGRES_EMPTY_QUERY, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_FATAL_ERROR", (void *)_cffi_const_PGRES_FATAL_ERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_NONFATAL_ERROR", (void *)_cffi_const_PGRES_NONFATAL_ERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_POLLING_ACTIVE", (void *)_cffi_const_PGRES_POLLING_ACTIVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_POLLING_FAILED", (void *)_cffi_const_PGRES_POLLING_FAILED, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_POLLING_OK", (void *)_cffi_const_PGRES_POLLING_OK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_POLLING_READING", (void *)_cffi_const_PGRES_POLLING_READING, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_POLLING_WRITING", (void *)_cffi_const_PGRES_POLLING_WRITING, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PGRES_TUPLES_OK", (void *)_cffi_const_PGRES_TUPLES_OK, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PQEgetdouble", (void *)_cffi_f_PQEgetdouble, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 121), (void *)_cffi_d_PQEgetdouble },
  { "PQEgetfloat", (void *)_cffi_f_PQEgetfloat, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 127), (void *)_cffi_d_PQEgetfloat },
  { "PQEgetint", (void *)_cffi_f_PQEgetint, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 133), (void *)_cffi_d_PQEgetint },
  { "PQEgetlong", (void *)_cffi_f_PQEgetlong, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 139), (void *)_cffi_d_PQEgetlong },
  { "PQTRANS_ACTIVE", (void *)_cffi_const_PQTRANS_ACTIVE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PQTRANS_IDLE", (void *)_cffi_const_PQTRANS_IDLE, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PQTRANS_INERROR", (void *)_cffi_const_PQTRANS_INERROR, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PQTRANS_INTRANS", (void *)_cffi_const_PQTRANS_INTRANS, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PQTRANS_UNKNOWN", (void *)_cffi_const_PQTRANS_UNKNOWN, _CFFI_OP(_CFFI_OP_ENUM, -1), (void *)0 },
  { "PQbackendPID", (void *)_cffi_f_PQbackendPID, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_PQbackendPID },
  { "PQcancel", (void *)_cffi_f_PQcancel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 43), (void *)_cffi_d_PQcancel },
  { "PQclear", (void *)_cffi_f_PQclear, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 193), (void *)_cffi_d_PQclear },
  { "PQcmdStatus", (void *)_cffi_f_PQcmdStatus, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 24), (void *)_cffi_d_PQcmdStatus },
  { "PQcmdTuples", (void *)_cffi_f_PQcmdTuples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 24), (void *)_cffi_d_PQcmdTuples },
  { "PQconnectPoll", (void *)_cffi_f_PQconnectPoll, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 48), (void *)_cffi_d_PQconnectPoll },
  { "PQconnectStart", (void *)_cffi_f_PQconnectStart, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 3), (void *)_cffi_d_PQconnectStart },
  { "PQconnectdb", (void *)_cffi_f_PQconnectdb, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 3), (void *)_cffi_d_PQconnectdb },
  { "PQconsumeInput", (void *)_cffi_f_PQconsumeInput, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 48), (void *)_cffi_d_PQconsumeInput },
  { "PQerrorMessage", (void *)_cffi_f_PQerrorMessage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 21), (void *)_cffi_d_PQerrorMessage },
  { "PQescapeBytea", (void *)_cffi_f_PQescapeBytea, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 167), (void *)_cffi_d_PQescapeBytea },
  { "PQescapeByteaConn", (void *)_cffi_f_PQescapeByteaConn, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 157), (void *)_cffi_d_PQescapeByteaConn },
  { "PQescapeLiteral", (void *)_cffi_f_PQescapeLiteral, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 16), (void *)_cffi_d_PQescapeLiteral },
  { "PQescapeString", (void *)_cffi_f_PQescapeString, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 152), (void *)_cffi_d_PQescapeString },
  { "PQescapeStringConn", (void *)_cffi_f_PQescapeStringConn, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 145), (void *)_cffi_d_PQescapeStringConn },
  { "PQexec", (void *)_cffi_f_PQexec, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 12), (void *)_cffi_d_PQexec },
  { "PQfinish", (void *)_cffi_f_PQfinish, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 190), (void *)_cffi_d_PQfinish },
  { "PQflush", (void *)_cffi_f_PQflush, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 48), (void *)_cffi_d_PQflush },
  { "PQfmod", (void *)_cffi_f_PQfmod, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 112), (void *)_cffi_d_PQfmod },
  { "PQfname", (void *)_cffi_f_PQfname, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 30), (void *)_cffi_d_PQfname },
  { "PQfreeCancel", (void *)_cffi_f_PQfreeCancel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 187), (void *)_cffi_d_PQfreeCancel },
  { "PQfreemem", (void *)_cffi_f_PQfreemem, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 196), (void *)_cffi_d_PQfreemem },
  { "PQfsize", (void *)_cffi_f_PQfsize, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 112), (void *)_cffi_d_PQfsize },
  { "PQftype", (void *)_cffi_f_PQftype, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 183), (void *)_cffi_d_PQftype },
  { "PQgetCancel", (void *)_cffi_f_PQgetCancel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 0), (void *)_cffi_d_PQgetCancel },
  { "PQgetCopyData", (void *)_cffi_f_PQgetCopyData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 51), (void *)_cffi_d_PQgetCopyData },
  { "PQgetResult", (void *)_cffi_f_PQgetResult, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 9), (void *)_cffi_d_PQgetResult },
  { "PQgetisnull", (void *)_cffi_f_PQgetisnull, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 116), (void *)_cffi_d_PQgetisnull },
  { "PQgetlength", (void *)_cffi_f_PQgetlength, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 116), (void *)_cffi_d_PQgetlength },
  { "PQgetvalue", (void *)_cffi_f_PQgetvalue, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 34), (void *)_cffi_d_PQgetvalue },
  { "PQisBusy", (void *)_cffi_f_PQisBusy, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 48), (void *)_cffi_d_PQisBusy },
  { "PQnfields", (void *)_cffi_f_PQnfields, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 109), (void *)_cffi_d_PQnfields },
  { "PQnotifies", (void *)_cffi_f_PQnotifies, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 6), (void *)_cffi_d_PQnotifies },
  { "PQntuples", (void *)_cffi_f_PQntuples, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 109), (void *)_cffi_d_PQntuples },
  { "PQoidValue", (void *)_cffi_f_PQoidValue, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 180), (void *)_cffi_d_PQoidValue },
  { "PQparameterStatus", (void *)_cffi_f_PQparameterStatus, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 39), (void *)_cffi_d_PQparameterStatus },
  { "PQprotocolVersion", (void *)_cffi_f_PQprotocolVersion, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_PQprotocolVersion },
  { "PQputCopyData", (void *)_cffi_f_PQputCopyData, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 60), (void *)_cffi_d_PQputCopyData },
  { "PQputCopyEnd", (void *)_cffi_f_PQputCopyEnd, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 56), (void *)_cffi_d_PQputCopyEnd },
  { "PQrequestCancel", (void *)_cffi_f_PQrequestCancel, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 48), (void *)_cffi_d_PQrequestCancel },
  { "PQresultErrorField", (void *)_cffi_f_PQresultErrorField, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 30), (void *)_cffi_d_PQresultErrorField },
  { "PQresultErrorMessage", (void *)_cffi_f_PQresultErrorMessage, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 27), (void *)_cffi_d_PQresultErrorMessage },
  { "PQresultStatus", (void *)_cffi_f_PQresultStatus, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 109), (void *)_cffi_d_PQresultStatus },
  { "PQsendQuery", (void *)_cffi_f_PQsendQuery, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 56), (void *)_cffi_d_PQsendQuery },
  { "PQserverVersion", (void *)_cffi_f_PQserverVersion, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_PQserverVersion },
  { "PQsetNoticeProcessor", (void *)_cffi_f_PQsetNoticeProcessor, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 203), (void *)_cffi_d_PQsetNoticeProcessor },
  { "PQsetnonblocking", (void *)_cffi_f_PQsetnonblocking, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 65), (void *)_cffi_d_PQsetnonblocking },
  { "PQsocket", (void *)_cffi_f_PQsocket, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_PQsocket },
  { "PQstatus", (void *)_cffi_f_PQstatus, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_PQstatus },
  { "PQtransactionStatus", (void *)_cffi_f_PQtransactionStatus, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_O, 106), (void *)_cffi_d_PQtransactionStatus },
  { "PQunescapeBytea", (void *)_cffi_f_PQunescapeBytea, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 163), (void *)_cffi_d_PQunescapeBytea },
  { "_PG_VERSION", (void *)_cffi_const__PG_VERSION, _CFFI_OP(_CFFI_OP_CONSTANT_INT, -1), (void *)0 },
  { "lo_close", (void *)_cffi_f_lo_close, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 65), (void *)_cffi_d_lo_close },
  { "lo_create", (void *)_cffi_f_lo_create, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 176), (void *)_cffi_d_lo_create },
  { "lo_export", (void *)_cffi_f_lo_export, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 96), (void *)_cffi_d_lo_export },
  { "lo_import", (void *)_cffi_f_lo_import, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 172), (void *)_cffi_d_lo_import },
  { "lo_lseek", (void *)_cffi_f_lo_lseek, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 81), (void *)_cffi_d_lo_lseek },
  { "lo_open", (void *)_cffi_f_lo_open, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 101), (void *)_cffi_d_lo_open },
  { "lo_read", (void *)_cffi_f_lo_read, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 69), (void *)_cffi_d_lo_read },
  { "lo_tell", (void *)_cffi_f_lo_tell, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 65), (void *)_cffi_d_lo_tell },
  { "lo_truncate", (void *)_cffi_f_lo_truncate, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 87), (void *)_cffi_d_lo_truncate },
  { "lo_unlink", (void *)_cffi_f_lo_unlink, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 92), (void *)_cffi_d_lo_unlink },
  { "lo_write", (void *)_cffi_f_lo_write, _CFFI_OP(_CFFI_OP_CPYTHON_BLTN_V, 75), (void *)_cffi_d_lo_write },
};

static const struct _cffi_field_s _cffi_fields[] = {
  { "relname", offsetof(PGnotify, relname),
               sizeof(((PGnotify *)0)->relname),
               _CFFI_OP(_CFFI_OP_NOOP, 45) },
  { "be_pid", offsetof(PGnotify, be_pid),
              sizeof(((PGnotify *)0)->be_pid),
              _CFFI_OP(_CFFI_OP_NOOP, 32) },
  { "extra", offsetof(PGnotify, extra),
             sizeof(((PGnotify *)0)->extra),
             _CFFI_OP(_CFFI_OP_NOOP, 45) },
};

static const struct _cffi_struct_union_s _cffi_struct_unions[] = {
  { "$PGcancel", 211, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "$PGconn", 212, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "$PGresult", 215, _CFFI_F_OPAQUE,
    (size_t)-1, -1, -1, 0 /* opaque */ },
  { "pgNotify", 214, 0,
    sizeof(PGnotify), offsetof(struct _cffi_align__PGnotify, y), 0, 3 },
};

static const struct _cffi_enum_s _cffi_enums[] = {
  { "$ConnStatusType", 208, _cffi_prim_int(sizeof(ConnStatusType), ((ConnStatusType)-1) <= 0),
    "CONNECTION_OK,CONNECTION_BAD,CONNECTION_STARTED,CONNECTION_MADE,CONNECTION_AWAITING_RESPONSE,CONNECTION_AUTH_OK,CONNECTION_SETENV,CONNECTION_SSL_STARTUP,CONNECTION_NEEDED" },
  { "$ExecStatusType", 209, _cffi_prim_int(sizeof(ExecStatusType), ((ExecStatusType)-1) <= 0),
    "PGRES_EMPTY_QUERY,PGRES_COMMAND_OK,PGRES_TUPLES_OK,PGRES_COPY_OUT,PGRES_COPY_IN,PGRES_BAD_RESPONSE,PGRES_NONFATAL_ERROR,PGRES_FATAL_ERROR" },
  { "$PGTransactionStatusType", 210, _cffi_prim_int(sizeof(PGTransactionStatusType), ((PGTransactionStatusType)-1) <= 0),
    "PQTRANS_IDLE,PQTRANS_ACTIVE,PQTRANS_INTRANS,PQTRANS_INERROR,PQTRANS_UNKNOWN" },
  { "$PostgresPollingStatusType", 216, _cffi_prim_int(sizeof(PostgresPollingStatusType), ((PostgresPollingStatusType)-1) <= 0),
    "PGRES_POLLING_FAILED,PGRES_POLLING_READING,PGRES_POLLING_WRITING,PGRES_POLLING_OK,PGRES_POLLING_ACTIVE" },
};

static const struct _cffi_typename_s _cffi_typenames[] = {
  { "ConnStatusType", 208 },
  { "ExecStatusType", 209 },
  { "Oid", 94 },
  { "PGTransactionStatusType", 210 },
  { "PGcancel", 211 },
  { "PGconn", 212 },
  { "PGnotify", 214 },
  { "PGresult", 215 },
  { "PQnoticeProcessor", 205 },
  { "PostgresPollingStatusType", 216 },
};

static const struct _cffi_type_context_s _cffi_type_context = {
  _cffi_types,
  _cffi_globals,
  _cffi_fields,
  _cffi_struct_unions,
  _cffi_enums,
  _cffi_typenames,
  111,  /* num_globals */
  4,  /* num_struct_unions */
  4,  /* num_enums */
  10,  /* num_typenames */
  NULL,  /* no includes */
  225,  /* num_types */
  0,  /* flags */
};

#ifdef __GNUC__
#  pragma GCC visibility push(default)  /* for -fvisibility= */
#endif

#ifdef PYPY_VERSION
PyMODINIT_FUNC
_cffi_pypyinit__libpq(const void *p[])
{
    p[0] = (const void *)0x2601;
    p[1] = &_cffi_type_context;
#if PY_MAJOR_VERSION >= 3
    return NULL;
#endif
}
#  ifdef _MSC_VER
     PyMODINIT_FUNC
#  if PY_MAJOR_VERSION >= 3
     PyInit__libpq(void) { return NULL; }
#  else
     init_libpq(void) { }
#  endif
#  endif
#elif PY_MAJOR_VERSION >= 3
PyMODINIT_FUNC
PyInit__libpq(void)
{
  return _cffi_init("psycopg2cffi._impl._libpq", 0x2601, &_cffi_type_context);
}
#else
PyMODINIT_FUNC
init_libpq(void)
{
  _cffi_init("psycopg2cffi._impl._libpq", 0x2601, &_cffi_type_context);
}
#endif

#ifdef __GNUC__
#  pragma GCC visibility pop
#endif
